@@ -106,18 +106,18 @@ describe OmniAuth::Strategy do
     end
 
     it 'sets args to the specified argument if there is one' do
-      subject.args [:abc, :def]
-      expect(subject.args).to eq([:abc, :def])
+      subject.args %i[abc def]
+      expect(subject.args).to eq(%i[abc def])
     end
 
     it 'is inheritable' do
-      subject.args [:abc, :def]
+      subject.args %i[abc def]
       c = Class.new(subject)
-      expect(c.args).to eq([:abc, :def])
+      expect(c.args).to eq(%i[abc def])
     end
 
     it 'accepts corresponding options as default arg values' do
-      subject.args [:a, :b]
+      subject.args %i[a b]
       subject.option :a, '1'
       subject.option :b, '2'
 
@@ -130,7 +130,7 @@ describe OmniAuth::Strategy do
 
   context 'fetcher procs' do
     subject { fresh_strategy }
-    %w(uid info credentials extra).each do |fetcher|
+    %w[uid info credentials extra].each do |fetcher|
       describe ".#{fetcher}" do
         it 'sets and retrieve a proc' do
           proc = lambda { 'Hello' }
@@ -143,7 +143,7 @@ describe OmniAuth::Strategy do
 
   context 'fetcher stacks' do
     subject { fresh_strategy }
-    %w(uid info credentials extra).each do |fetcher|
+    %w[uid info credentials extra].each do |fetcher|
       describe ".#{fetcher}_stack" do
         it 'is an array of called ancestral procs' do
           fetchy = proc { 'Hello' }
@@ -154,7 +154,7 @@ describe OmniAuth::Strategy do
     end
   end
 
-  %w(request_phase).each do |abstract_method|
+  %w[request_phase].each do |abstract_method|
     context abstract_method.to_s do
       it 'raises a NotImplementedError' do
         strat = Class.new
@@ -208,7 +208,7 @@ describe OmniAuth::Strategy do
       end
 
       it 'sets options based on the arguments if they are supplied' do
-        subject.args [:abc, :def]
+        subject.args %i[abc def]
         s = subject.new app, 123, 456
         expect(s.options[:abc]).to eq(123)
         expect(s.options[:def]).to eq(456)
@@ -224,6 +224,13 @@ describe OmniAuth::Strategy do
       expect(instance).to receive(:dup).and_return(instance)
       instance.call('rack.session' => {})
     end
+
+    it 'raises NoSessionError if rack.session isn\'t set' do
+      klass = Class.new
+      klass.send :include, OmniAuth::Strategy
+      instance = klass.new(app)
+      expect { instance.call({}) }.to raise_error(OmniAuth::NoSessionError)
+    end
   end
 
   describe '#inspect' do
@@ -277,7 +284,7 @@ describe OmniAuth::Strategy do
     end
   end
 
-  %w(info credentials extra).each do |fetcher|
+  %w[info credentials extra].each do |fetcher|
     subject { fresh_strategy }
     it "is the current class's proc call if one exists" do
       subject.send(fetcher) { {:abc => 123} }
@@ -300,42 +307,60 @@ describe OmniAuth::Strategy do
     let(:strategy) { ExampleStrategy.new(app, @options || {}) }
 
     context 'omniauth.origin' do
-      it 'is set on the request phase' do
-        expect { strategy.call(make_env('/auth/test', 'HTTP_REFERER' => 'http://example.com/origin')) }.to raise_error('Request Phase')
-        expect(strategy.last_env['rack.session']['omniauth.origin']).to eq('http://example.com/origin')
+      context 'disabled' do
+        it 'does not set omniauth.origin' do
+          @options = {:origin_param => false}
+          expect { strategy.call(make_env('/auth/test', 'QUERY_STRING' => 'return=/foo')) }.to raise_error('Request Phase')
+          expect(strategy.last_env['rack.session']['omniauth.origin']).to eq(nil)
+        end
       end
 
-      it 'is turned into an env variable on the callback phase' do
-        expect { strategy.call(make_env('/auth/test/callback', 'rack.session' => {'omniauth.origin' => 'http://example.com/origin'})) }.to raise_error('Callback Phase')
-        expect(strategy.last_env['omniauth.origin']).to eq('http://example.com/origin')
+      context 'custom' do
+        it 'sets from a custom param' do
+          @options = {:origin_param => 'return'}
+          expect { strategy.call(make_env('/auth/test', 'QUERY_STRING' => 'return=/foo')) }.to raise_error('Request Phase')
+          expect(strategy.last_env['rack.session']['omniauth.origin']).to eq('/foo')
+        end
       end
 
-      it 'sets from the params if provided' do
-        expect { strategy.call(make_env('/auth/test', 'QUERY_STRING' => 'origin=/foo')) }.to raise_error('Request Phase')
-        expect(strategy.last_env['rack.session']['omniauth.origin']).to eq('/foo')
-      end
-
-      it 'is set on the failure env' do
-        expect(OmniAuth.config).to receive(:on_failure).and_return(lambda { |env| env })
-        @options = {:failure => :forced_fail}
-        strategy.call(make_env('/auth/test/callback', 'rack.session' => {'omniauth.origin' => '/awesome'}))
-      end
-
-      context 'with script_name' do
-        it 'is set on the request phase, containing full path' do
-          env = {'HTTP_REFERER' => 'http://example.com/sub_uri/origin', 'SCRIPT_NAME' => '/sub_uri'}
-          expect { strategy.call(make_env('/auth/test', env)) }.to raise_error('Request Phase')
-          expect(strategy.last_env['rack.session']['omniauth.origin']).to eq('http://example.com/sub_uri/origin')
+      context 'default flow' do
+        it 'is set on the request phase' do
+          expect { strategy.call(make_env('/auth/test', 'HTTP_REFERER' => 'http://example.com/origin')) }.to raise_error('Request Phase')
+          expect(strategy.last_env['rack.session']['omniauth.origin']).to eq('http://example.com/origin')
         end
 
-        it 'is turned into an env variable on the callback phase, containing full path' do
-          env = {
-            'rack.session' => {'omniauth.origin' => 'http://example.com/sub_uri/origin'},
-            'SCRIPT_NAME' => '/sub_uri'
-          }
+        it 'is turned into an env variable on the callback phase' do
+          expect { strategy.call(make_env('/auth/test/callback', 'rack.session' => {'omniauth.origin' => 'http://example.com/origin'})) }.to raise_error('Callback Phase')
+          expect(strategy.last_env['omniauth.origin']).to eq('http://example.com/origin')
+        end
 
-          expect { strategy.call(make_env('/auth/test/callback', env)) }.to raise_error('Callback Phase')
-          expect(strategy.last_env['omniauth.origin']).to eq('http://example.com/sub_uri/origin')
+        it 'sets from the params if provided' do
+          expect { strategy.call(make_env('/auth/test', 'QUERY_STRING' => 'origin=/foo')) }.to raise_error('Request Phase')
+          expect(strategy.last_env['rack.session']['omniauth.origin']).to eq('/foo')
+        end
+
+        it 'is set on the failure env' do
+          expect(OmniAuth.config).to receive(:on_failure).and_return(lambda { |env| env })
+          @options = {:failure => :forced_fail}
+          strategy.call(make_env('/auth/test/callback', 'rack.session' => {'omniauth.origin' => '/awesome'}))
+        end
+
+        context 'with script_name' do
+          it 'is set on the request phase, containing full path' do
+            env = {'HTTP_REFERER' => 'http://example.com/sub_uri/origin', 'SCRIPT_NAME' => '/sub_uri'}
+            expect { strategy.call(make_env('/auth/test', env)) }.to raise_error('Request Phase')
+            expect(strategy.last_env['rack.session']['omniauth.origin']).to eq('http://example.com/sub_uri/origin')
+          end
+
+          it 'is turned into an env variable on the callback phase, containing full path' do
+            env = {
+              'rack.session' => {'omniauth.origin' => 'http://example.com/sub_uri/origin'},
+              'SCRIPT_NAME' => '/sub_uri'
+            }
+
+            expect { strategy.call(make_env('/auth/test/callback', env)) }.to raise_error('Callback Phase')
+            expect(strategy.last_env['omniauth.origin']).to eq('http://example.com/sub_uri/origin')
+          end
         end
       end
     end
@@ -510,7 +535,7 @@ describe OmniAuth::Strategy do
       end
 
       after do
-        OmniAuth.config.allowed_request_methods = [:get, :post]
+        OmniAuth.config.allowed_request_methods = %i[get post]
       end
     end
 
@@ -664,14 +689,34 @@ describe OmniAuth::Strategy do
         expect(strategy.env['omniauth.error.type']).to eq(:invalid_credentials)
       end
 
-      it 'sets omniauth.origin on the request phase' do
-        strategy.call(make_env('/auth/test', 'HTTP_REFERER' => 'http://example.com/origin'))
-        expect(strategy.env['rack.session']['omniauth.origin']).to eq('http://example.com/origin')
-      end
+      context 'omniauth.origin' do
+        context 'disabled' do
+          it 'does not set omniauth.origin' do
+            @options = {:origin_param => false}
+            strategy.call(make_env('/auth/test', 'HTTP_REFERER' => 'http://example.com/origin'))
+            expect(strategy.env['rack.session']['omniauth.origin']).to be_nil
+          end
+        end
 
-      it 'sets omniauth.origin from the params if provided' do
-        strategy.call(make_env('/auth/test', 'QUERY_STRING' => 'origin=/foo'))
-        expect(strategy.env['rack.session']['omniauth.origin']).to eq('/foo')
+        context 'default flow' do
+          it 'sets omniauth.origin to the HTTP_REFERER on the request phase by default' do
+            strategy.call(make_env('/auth/test', 'HTTP_REFERER' => 'http://example.com/origin'))
+            expect(strategy.env['rack.session']['omniauth.origin']).to eq('http://example.com/origin')
+          end
+
+          it 'sets omniauth.origin from the params if provided' do
+            strategy.call(make_env('/auth/test', 'QUERY_STRING' => 'origin=/foo'))
+            expect(strategy.env['rack.session']['omniauth.origin']).to eq('/foo')
+          end
+        end
+
+        context 'custom' do
+          it 'sets omniauth.origin from a custom param' do
+            @options = {:origin_param => 'return'}
+            strategy.call(make_env('/auth/test', 'QUERY_STRING' => 'return=/foo'))
+            expect(strategy.env['rack.session']['omniauth.origin']).to eq('/foo')
+          end
+        end
       end
 
       it 'turns omniauth.origin into an env variable on the callback phase' do
